@@ -1,8 +1,8 @@
 /*
  * integritysetup - setup integrity protected volumes for dm-integrity
  *
- * Copyright (C) 2017-2020 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2017-2020 Milan Broz
+ * Copyright (C) 2017-2021 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2017-2021 Milan Broz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,10 +25,20 @@
 #define PACKAGE_INTEGRITY "integritysetup"
 
 #define DEFAULT_ALG_NAME "crc32c"
-#define MAX_KEY_SIZE 4096
 
-static const char *opt_journal_size_str = NULL;
+static char *opt_data_device = NULL;
+static char *opt_integrity = NULL; /* DEFAULT_ALG_NAME */
+static char *opt_integrity_key_file = NULL;
+static char *opt_journal_integrity = NULL; /* none */
+static char *opt_journal_integrity_key_file = NULL;
+static char *opt_journal_crypt = NULL; /* none */
+static char *opt_journal_crypt_key_file = NULL;
+
+/* helper strings converted to uint64_t later */
+static char *opt_journal_size_str = NULL;
+
 static uint64_t opt_journal_size = 0;
+
 static int opt_interleave_sectors = 0;
 static int opt_journal_watermark = 0;
 static int opt_bitmap_sectors_per_bit = 0;
@@ -37,40 +47,42 @@ static int opt_bitmap_flush_time = 0;
 static int opt_tag_size = 0;
 static int opt_sector_size = 0;
 static int opt_buffer_sectors = 0;
-
 static int opt_no_wipe = 0;
-
-static const char *opt_data_device = NULL;
-
-static const char *opt_integrity = DEFAULT_ALG_NAME;
-static const char *opt_integrity_key_file = NULL;
 static int opt_integrity_key_size = 0;
-
-static const char *opt_journal_integrity = NULL; /* none */
-static const char *opt_journal_integrity_key_file = NULL;
 static int opt_journal_integrity_key_size = 0;
-
-static const char *opt_journal_crypt = NULL; /* none */
-static const char *opt_journal_crypt_key_file = NULL;
 static int opt_journal_crypt_key_size = 0;
-
 static int opt_integrity_nojournal = 0;
 static int opt_integrity_recovery = 0;
 static int opt_integrity_bitmap = 0;
 static int opt_integrity_legacy_padding = 0;
-
+static int opt_integrity_legacy_hmac = 0;
+static int opt_integrity_legacy_recalculate = 0;
 static int opt_integrity_recalculate = 0;
+static int opt_allow_discards = 0;
 
+static const char *integrity_alg = DEFAULT_ALG_NAME;
 static const char **action_argv;
 static int action_argc;
 
+void tools_cleanup(void)
+{
+	FREE_AND_NULL(opt_data_device);
+	FREE_AND_NULL(opt_integrity);
+	FREE_AND_NULL(opt_integrity_key_file);
+	FREE_AND_NULL(opt_journal_integrity);
+	FREE_AND_NULL(opt_journal_integrity_key_file);
+	FREE_AND_NULL(opt_journal_crypt);
+	FREE_AND_NULL(opt_journal_crypt_key_file);
+	FREE_AND_NULL(opt_journal_size_str);
+}
+
 // FIXME: move this to tools and handle EINTR
 static int _read_mk(const char *file, char **key, int keysize)
 {
 	int fd;
 
-	if (keysize <= 0 || keysize > MAX_KEY_SIZE) {
-		log_err(_("Invalid key size."));
+	if (keysize <= 0 || keysize > (DEFAULT_INTEGRITY_KEYFILE_SIZE_MAXKB * 1024)) {
+		log_err(_("Invalid key size. Maximum is %u bytes."), DEFAULT_INTEGRITY_KEYFILE_SIZE_MAXKB * 1024);
 		return -EINVAL;
 	}
 
@@ -188,14 +200,12 @@ static int action_format(int arg)
 	int r;
 	size_t signatures;
 
-	if (opt_integrity) {
-		r = crypt_parse_hash_integrity_mode(opt_integrity, integrity);
-		if (r < 0) {
-			log_err(_("No known integrity specification pattern detected."));
-			return r;
-		}
-		params.integrity = integrity;
+	r = crypt_parse_hash_integrity_mode(integrity_alg, integrity);
+	if (r < 0) {
+		log_err(_("No known integrity specification pattern detected."));
+		return r;
 	}
+	params.integrity = integrity;
 
 	if (opt_journal_integrity) {
 		r = crypt_parse_hash_integrity_mode(opt_journal_integrity, journal_integrity);
@@ -245,6 +255,9 @@ static int action_format(int arg)
 	if (opt_integrity_legacy_padding)
 		crypt_set_compatibility(cd, CRYPT_COMPAT_LEGACY_INTEGRITY_PADDING);
 
+	if (opt_integrity_legacy_hmac)
+		crypt_set_compatibility(cd, CRYPT_COMPAT_LEGACY_INTEGRITY_HMAC);
+
 	r = crypt_format(cd, CRYPT_INTEGRITY, NULL, NULL, NULL, NULL, 0, &params);
 	if (r < 0) /* FIXME: call wipe signatures again */
 		goto out;
@@ -277,14 +290,12 @@ static int action_open(int arg)
 	char *integrity_key = NULL;
 	int r;
 
-	if (opt_integrity) {
-		r = crypt_parse_hash_integrity_mode(opt_integrity, integrity);
-		if (r < 0) {
-			log_err(_("No known integrity specification pattern detected."));
-			return r;
-		}
-		params.integrity = integrity;
+	r = crypt_parse_hash_integrity_mode(integrity_alg, integrity);
+	if (r < 0) {
+		log_err(_("No known integrity specification pattern detected."));
+		return r;
 	}
+	params.integrity = integrity;
 
 	if (opt_journal_integrity) {
 		r = crypt_parse_hash_integrity_mode(opt_journal_integrity, journal_integrity);
@@ -312,8 +323,10 @@ static int action_open(int arg)
 	if (opt_integrity_bitmap)
 		activate_flags |= CRYPT_ACTIVATE_NO_JOURNAL_BITMAP;
 
-	if (opt_integrity_recalculate)
+	if (opt_integrity_recalculate || opt_integrity_legacy_recalculate)
 		activate_flags |= CRYPT_ACTIVATE_RECALCULATE;
+	if (opt_allow_discards)
+		activate_flags |= CRYPT_ACTIVATE_ALLOW_DISCARDS;
 
 	r = _read_keys(&integrity_key, &params);
 	if (r)
@@ -326,6 +339,9 @@ static int action_open(int arg)
 	if (r)
 		goto out;
 
+	if (opt_integrity_legacy_recalculate)
+		crypt_set_compatibility(cd, CRYPT_COMPAT_LEGACY_INTEGRITY_RECALC);
+
 	r = crypt_activate_by_volume_key(cd, action_argv[1], integrity_key,
 					 opt_integrity_key_size, activate_flags);
 out:
@@ -437,6 +453,9 @@ static int action_status(int arg)
 			if (ip.journal_crypt)
 				log_std("  journal encryption: %s\n", ip.journal_crypt);
 		}
+		if (cad.flags & (CRYPT_ACTIVATE_ALLOW_DISCARDS))
+			log_std("  flags: %s\n",
+				(cad.flags & CRYPT_ACTIVATE_ALLOW_DISCARDS) ? "discards " : "");
 	}
 out:
 	crypt_free(cd);
@@ -499,11 +518,15 @@ static void help(poptContext popt_context,
 			crypt_get_dir());
 
 		log_std(_("\nDefault compiled-in dm-integrity parameters:\n"
-			  "\tChecksum algorithm: %s\n"), DEFAULT_ALG_NAME);
+			  "\tChecksum algorithm: %s\n"
+			  "\tMaximum keyfile size: %dkB\n"),
+			  DEFAULT_ALG_NAME, DEFAULT_INTEGRITY_KEYFILE_SIZE_MAXKB);
+		tools_cleanup();
 		poptFreeContext(popt_context);
 		exit(EXIT_SUCCESS);
 	} else if (key->shortName == 'V') {
 		log_std("%s %s\n", PACKAGE_INTEGRITY, PACKAGE_VERSION);
+		tools_cleanup();
 		poptFreeContext(popt_context);
 		exit(EXIT_SUCCESS);
 	} else
@@ -569,6 +592,11 @@ int main(int argc, const char **argv)
 		{ "integrity-bitmap-mode",      'B', POPT_ARG_NONE,  &opt_integrity_bitmap, 0, N_("Use bitmap to track changes and disable journal for integrity device"), NULL },
 		{ "integrity-recalculate",     '\0', POPT_ARG_NONE,  &opt_integrity_recalculate,  0, N_("Recalculate initial tags automatically."), NULL },
 		{ "integrity-legacy-padding",  '\0', POPT_ARG_NONE,  &opt_integrity_legacy_padding, 0, N_("Use inefficient legacy padding (old kernels)"), NULL },
+
+		{ "integrity-legacy-hmac",     '\0', POPT_ARG_NONE,  &opt_integrity_legacy_hmac, 0, N_("Do not protect superblock with HMAC (old kernels)"), NULL },
+		{ "integrity-legacy-recalculate",'\0',POPT_ARG_NONE, &opt_integrity_legacy_recalculate, 0, N_("Allow recalculating of volumes with HMAC keys (old kernels)"), NULL },
+
+		{ "allow-discards",            '\0', POPT_ARG_NONE,  &opt_allow_discards, 0, N_("Allow discards (aka TRIM) requests for device"), NULL },
 		POPT_TABLEEND
 	};
 	poptContext popt_context;
@@ -621,6 +649,9 @@ int main(int argc, const char **argv)
 		aname = "close";
 	}
 
+	if (opt_integrity)
+		integrity_alg = opt_integrity;
+
 	for (action = action_types; action->type; action++)
 		if (strcmp(action->type, aname) == 0)
 			break;
@@ -641,6 +672,11 @@ int main(int argc, const char **argv)
 		      _("Option --integrity-recalculate can be used only for open action."),
 		      poptGetInvocationName(popt_context));
 
+	if (opt_allow_discards && strcmp(aname, "open"))
+		usage(popt_context, EXIT_FAILURE,
+		      _("Option --allow-discards is allowed only for open operation."),
+		      poptGetInvocationName(popt_context));
+
 	if (opt_interleave_sectors < 0 || opt_journal_watermark < 0 ||
 	    opt_journal_commit_time < 0 || opt_tag_size < 0 ||
 	    opt_sector_size < 0 || opt_buffer_sectors < 0 ||
@@ -654,7 +690,7 @@ int main(int argc, const char **argv)
 		opt_sector_size || opt_tag_size || opt_no_wipe ))
 		usage(popt_context, EXIT_FAILURE,
 		      _("Options --journal-size, --interleave-sectors, --sector-size, --tag-size"
-		        " and --no-wipe can be used only for format action.\n"),
+		        " and --no-wipe can be used only for format action."),
 		      poptGetInvocationName(popt_context));
 
 	if (opt_journal_size_str &&
@@ -666,9 +702,6 @@ int main(int argc, const char **argv)
 	   (!opt_integrity_key_file && opt_integrity_key_size))
 		usage(popt_context, EXIT_FAILURE, _("Both key file and key size options must be specified."),
 		      poptGetInvocationName(popt_context));
-	if (!opt_integrity && opt_integrity_key_file)
-		usage(popt_context, EXIT_FAILURE, _("Integrity algorithm must be specified if integrity key is used."),
-		      poptGetInvocationName(popt_context));
 
 	if ((opt_journal_integrity_key_file && !opt_journal_integrity_key_size) ||
 	   (!opt_journal_integrity_key_file && opt_journal_integrity_key_size))
@@ -705,6 +738,7 @@ int main(int argc, const char **argv)
 	}
 
 	r = run_action(action);
+	tools_cleanup();
 	poptFreeContext(popt_context);
 	return r;
 }
