@@ -3,8 +3,8 @@
  *
  * Copyright (C) 2004 Jana Saout <jana@saout.de>
  * Copyright (C) 2004-2007 Clemens Fruhwirth <clemens@endorphin.org>
- * Copyright (C) 2009-2019 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2009-2019 Milan Broz
+ * Copyright (C) 2009-2021 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2009-2021 Milan Broz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -142,7 +142,7 @@ static int _dialog(const char *msg, void *usrptr, int default_answer)
 
 	if (isatty(STDIN_FILENO) && !opt_batch_mode) {
 		log_std("\nWARNING!\n========\n");
-		log_std("%s\n\nAre you sure? (Type uppercase yes): ", msg);
+		log_std("%s\n\nAre you sure? (Type 'yes' in capital letters): ", msg);
 		fflush(stdout);
 		if(getline(&answer, &size, stdin) == -1) {
 			r = 0;
@@ -238,6 +238,7 @@ __attribute__ ((noreturn)) void usage(poptContext popt_context,
 	poptPrintUsage(popt_context, stderr, 0);
 	if (error)
 		log_err("%s: %s", more, error);
+	tools_cleanup();
 	poptFreeContext(popt_context);
 	exit(exitcode);
 }
@@ -380,14 +381,14 @@ void tools_clear_line(void)
 	log_std("\33[2K\r");
 }
 
-void tools_time_progress(uint64_t device_size, uint64_t bytes,
+static void tools_time_progress(uint64_t device_size, uint64_t bytes, uint64_t *start_bytes,
 			 struct timeval *start_time, struct timeval *end_time)
 {
 	struct timeval now_time;
 	unsigned long long mbytes, eta;
-	double tdiff, mib, frequency;
+	double tdiff, uib, frequency;
 	int final = (bytes == device_size);
-	const char *eol;
+	const char *eol, *ustr = "";
 
 	if (opt_batch_mode)
 		return;
@@ -396,6 +397,7 @@ void tools_time_progress(uint64_t device_size, uint64_t bytes,
 	if (start_time->tv_sec == 0 && start_time->tv_usec == 0) {
 		*start_time = now_time;
 		*end_time = now_time;
+		*start_bytes = bytes;
 		return;
 	}
 
@@ -417,40 +419,50 @@ void tools_time_progress(uint64_t device_size, uint64_t bytes,
 		return;
 
 	mbytes = bytes  / 1024 / 1024;
-	mib = (double)(mbytes) / tdiff;
-	if (!mib)
-		return;
+	uib = (double)(bytes - *start_bytes) / tdiff;
 
-	/* FIXME: calculate this from last minute only and remaining space */
-	eta = (unsigned long long)(device_size / 1024 / 1024 / mib - tdiff);
+	/* FIXME: calculate this from last minute only. */
+	eta = (unsigned long long)(device_size / uib - tdiff);
+
+	if (uib > 1073741824.0f) {
+		uib /= 1073741824.0f;
+		ustr = "Gi";
+	} else if (uib > 1048576.0f) {
+		uib /= 1048576.0f;
+		ustr = "Mi";
+	} else if (uib > 1024.0f) {
+		uib /= 1024.0f;
+		ustr = "Ki";
+	}
 
 	tools_clear_line();
 	if (final)
 		log_std("Finished, time %02llu:%02llu.%03llu, "
-			"%4llu MiB written, speed %5.1f MiB/s\n",
+			"%4llu MiB written, speed %5.1f %sB/s\n",
 			(unsigned long long)tdiff / 60,
 			(unsigned long long)tdiff % 60,
 			(unsigned long long)((tdiff - floor(tdiff)) * 1000.0),
-			mbytes, mib);
+			mbytes, uib, ustr);
 	else
 		log_std("Progress: %5.1f%%, ETA %02llu:%02llu, "
-			"%4llu MiB written, speed %5.1f MiB/s%s",
+			"%4llu MiB written, speed %5.1f %sB/s%s",
 			(double)bytes / device_size * 100,
-			eta / 60, eta % 60, mbytes, mib, eol);
+			eta / 60, eta % 60, mbytes, uib, ustr, eol);
 	fflush(stdout);
 }
 
 int tools_wipe_progress(uint64_t size, uint64_t offset, void *usrptr)
 {
 	static struct timeval start_time = {}, end_time = {};
+	static uint64_t start_offset = 0;
 	int r = 0;
 
-	tools_time_progress(size, offset, &start_time, &end_time);
+	tools_time_progress(size, offset, &start_offset, &start_time, &end_time);
 
 	check_signal(&r);
 	if (r) {
 		tools_clear_line();
-		log_err("\nWipe interrupted.");
+		log_err(_("\nWipe interrupted."));
 	}
 
 	return r;
@@ -563,11 +575,11 @@ int tools_wipe_all_signatures(const char *path)
 
 	while ((pr = blk_probe(h)) < PRB_EMPTY) {
 		if (blk_is_partition(h))
-			log_verbose("Existing '%s' partition signature on device %s will be wiped.",
-				    blk_get_partition_type(h), path);
+			log_verbose(_("Existing '%s' partition signature (offset: %" PRIi64 " bytes) on device %s will be wiped."),
+				    blk_get_partition_type(h), blk_get_offset(h), path);
 		if (blk_is_superblock(h))
-			log_verbose("Existing '%s' superblock signature on device %s will be wiped.",
-				    blk_get_superblock_type(h), path);
+			log_verbose(_("Existing '%s' superblock signature (offset: %" PRIi64 " bytes) on device %s will be wiped."),
+				    blk_get_superblock_type(h), blk_get_offset(h), path);
 		if (blk_do_wipe(h)) {
 			log_err(_("Failed to wipe device signature."));
 			r = -EINVAL;
@@ -585,14 +597,6 @@ out:
 	return r;
 }
 
-int tools_is_cipher_null(const char *cipher)
-{
-	if (!cipher)
-		return 0;
-
-	return !strcmp(cipher, "cipher_null") ? 1 : 0;
-}
-
 /*
  * Keyfile - is standard input treated as a binary file (no EOL handling).
  */
@@ -607,14 +611,15 @@ int tools_is_stdin(const char *key_file)
 int tools_reencrypt_progress(uint64_t size, uint64_t offset, void *usrptr)
 {
 	static struct timeval start_time = {}, end_time = {};
+	static uint64_t start_offset = 0;
 	int r = 0;
 
-	tools_time_progress(size, offset, &start_time, &end_time);
+	tools_time_progress(size, offset, &start_offset, &start_time, &end_time);
 
 	check_signal(&r);
 	if (r) {
 		tools_clear_line();
-		log_err("\nReencrypt interrupted.");
+		log_err(_("\nReencryption interrupted."));
 	}
 
 	return r;
