@@ -1,8 +1,8 @@
 /*
  * cryptsetup library API test utilities
  *
- * Copyright (C) 2009-2020 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2009-2020 Milan Broz
+ * Copyright (C) 2009-2021 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2009-2021 Milan Broz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -95,6 +95,14 @@ void check_equal(int line, const char *func, int64_t x, int64_t y)
 	exit(-1);
 }
 
+void check_ge_equal(int line, const char *func, int64_t x, int64_t y)
+{
+	printf("FAIL line %d [%s]: expected greater or equal values differs: %"
+		PRIi64 " < %" PRIi64 "\n", line, func, x, y);
+	_cleanup();
+	exit(-1);
+}
+
 void check_null(int line, const char *func, const void *x)
 {
 	if (x) {
